"""Wea Schema"""
from pydantic import Field, constr, confloat
from typing import List
import datetime

from .._base import NoExtraBaseModel


class Location(NoExtraBaseModel):
    """Used to specify latitude and longitude of a location on the Earth."""

    type: constr(regex='^Location$') = 'Location'

    latitude: float = Field(
        ...,
        ge=-90,
        le=90,
        description='Location latitude between -90 and 90.'
    )

    longitude: float = Field(
        ...,
        ge=-180,
        le=180,
        description='Location longitude between -180 (west) and 180 (east).'
    )

    time_zone: float = Field(
        default=None,
        ge=-12,
        le=14,
        description='Time zone between -12 hours (west) and +14 hours (east). '
        'If None, the time zone will be an estimated integer value derived from '
        'the longitude in accordance with solar time.'
    )

    elevation: float = Field(
        default=0,
        description='A number for elevation of the location in meters.'
    )

    city: str = Field(
        default='-',
        description='Name of the city as a string.'
    )

    state: str = Field(
        default='-',
        description='Optional state in which the city is located.'
    )

    country: str = Field(
        default='-',
        description='Name of the country as a string.'
    )

    station_id: str = Field(
            default=None,
            description='Optional ID of the location if the location is '
            'representing a weather station.'
        )

    source: str = Field(
            default=None,
            description='Optional source of data (e.g. TMY, TMY3).'
        )


class Wea(NoExtraBaseModel):
    """Used to represent the contents of a Wea file."""

    type: constr(regex='^Wea$') = 'Wea'

    location: Location = Field(
        ...,
        description='Location object to note latitude, longitude and time zone.'
    )

    direct_normal_irradiance: List[confloat(ge=0)] = Field(
        ...,
        description='A list of numbers for the annual direct normal irradiance '
        'in W/m2.'
    )

    diffuse_horizontal_irradiance: List[confloat(ge=0)] = Field(
        ...,
        description='A list of numbers for the annual diffuse horizontal irradiance '
        'in W/m2.'
    )

    timestep: int = Field(
        1,
        ge=0,
        description='An integer to set the number of time steps per hour.'
    )

    is_leap_year: bool = Field(
        False,
        description='A boolean to indicate if values are representing a leap year.'
    )



# coding=utf-8
from __future__ import division

from ladybug.wea import Wea

from honeybee_radiance.lightsource.sky import CertainIrradiance, CIE, \
    ClimateBased, SunMatrix, SkyMatrix

import os
import json


def sky_certian_irradiance(directory):
    sky = CertainIrradiance.from_illuminance(10000)
    dest_file = os.path.join(directory, 'sky_certian_irradiance.json')
    with open(dest_file, 'w') as fp:
        json.dump(sky.to_dict(), fp, indent=4)


def sky_cie(directory):
    sky = CIE(38.186734, 270.410387)
    dest_file = os.path.join(directory, 'sky_cie.json')
    with open(dest_file, 'w') as fp:
        json.dump(sky.to_dict(), fp, indent=4)


def sky_climate_based(directory):
    sky = ClimateBased(38.186734, 270.410387, 702, 225)
    dest_file = os.path.join(directory, 'sky_climate_based.json')
    with open(dest_file, 'w') as fp:
        json.dump(sky.to_dict(), fp, indent=4)


def sun_matrix(directory):
    wea = './scripts/wea/denver.wea'
    wea = Wea.from_file(wea)
    sun_mtx = SunMatrix(wea)
    sun_mtx.north = 10
    dest_file = os.path.join(directory, 'sun_matrix.json')
    with open(dest_file, 'w') as fp:
        json.dump(sun_mtx.to_dict(), fp, indent=4)


def sky_matrix(directory):
    wea = './scripts/wea/denver.wea'
    wea = Wea.from_file(wea)
    sky_mtx = SkyMatrix(wea)
    sky_mtx.north = 10
    dest_file = os.path.join(directory, 'sky_matrix.json')
    with open(dest_file, 'w') as fp:
        json.dump(sky_mtx.to_dict(), fp, indent=4)


# run all functions within the file
master_dir = os.path.split(os.path.dirname(__file__))[0]
sample_directory = os.path.join(master_dir, 'samples', 'light_source')

sky_certian_irradiance(sample_directory)
sky_cie(sample_directory)
sky_climate_based(sample_directory)
sun_matrix(sample_directory)
sky_matrix(sample_directory)




from honeybee_schema.radiance.lightsource import CertainIrradiance, CIE, \
    ClimateBased, SunMatrix, SkyMatrix

import pytest
import os

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'light_source')


def test_sky_certian_irradiance():
    file_path = os.path.join(target_folder, 'sky_certian_irradiance.json')
    CertainIrradiance.parse_file(file_path)


def test_sky_cie():
    file_path = os.path.join(target_folder, 'sky_cie.json')
    CIE.parse_file(file_path)


def test_sky_climate_based():
    file_path = os.path.join(target_folder, 'sky_climate_based.json')
    ClimateBased.parse_file(file_path)


def test_sky_matrix():
    file_path = os.path.join(target_folder, 'sky_matrix.json')
    SkyMatrix.parse_file(file_path)


def test_sun_matrix():
    file_path = os.path.join(target_folder, 'sun_matrix.json')
    SunMatrix.parse_file(file_path)



"""SensorGrid and Sensor Schema"""
from pydantic import Field, constr
from typing import Optional

from .._base import NoExtraBaseModel
from .wea import Wea


class _GlowHemisphere(NoExtraBaseModel):
    """Hidden base class for Ground and SkyHemisphere."""

    r_emittance: float = Field(
        default=1.0,
        ge=0,
        le=1,
        description='A value between 0 and 1 for the red channel emittance.'
    )

    g_emittance: float = Field(
        default=1.0,
        ge=0,
        le=1,
        description='A value between 0 and 1 for the green channel emittance.'
    )

    b_emittance: float = Field(
        default=1.0,
        ge=0,
        le=1,
        description='A value between 0 and 1 for the blue channel of the emittance.'
    )


class Ground(_GlowHemisphere):
    """Ground component of the sky sphere."""

    type: constr(regex='^Ground$') = 'Ground'


class SkyHemisphere(_GlowHemisphere):
    """SkyHemisphere component of the sky sphere."""

    type: constr(regex='^SkyHemisphere$') = 'SkyHemisphere'


class SkyDome(NoExtraBaseModel):
    """Base class for all sky domes."""

    type: constr(regex='^SkyDome$') = 'SkyDome'

    ground_hemisphere: Optional[Ground] = Field(
        default=Ground(),
        description='Optional ground glow source.'
        )

    sky_hemisphere: Optional[SkyHemisphere] = Field(
        default=SkyHemisphere(),
        description='Optional sky hemisphere glow source.'
        )


class _PointInTime(SkyDome):
    """Hidden base class for all point-in-time sky domes."""

    ground_reflectance: float = Field(
        default=0.2,
        ge=0,
        le=1,
        description='A value between 0 and 1 for the ground reflectance.'
    )


class _SkyWithSun(_PointInTime):
    """Hidden base class for all altitude/azimuth sky domes."""

    altitude: float = Field(
        ...,
        ge=-90,
        le=90,
        description='The solar altitude measured in degrees above the horizon.'
        'Negative values indicate cases where the sun is below the horizon '
        '(eg. twilight conditions).'
    )

    azimuth: float = Field(
        ...,
        ge=0,
        le=360,
        description='The solar altitude measured in degrees above the horizon.'
        'The azimuth is measured in degrees east of North. East is 90, South is 180 and '
        'West is 270. Note that this input is different from Radiance convention. In '
        'Radiance the azimuth degrees are measured in west of South.'
    )


class CertainIrradiance(_PointInTime):
    """Sky with evenly distributed light at a certain irradiance value."""

    type: constr(regex='^CertainIrradiance$') = 'CertainIrradiance'

    irradiance: float = Field(
        default=558.659,
        ge=0,
        description='A value for the horizontal diffuse irradiance value in W/m2.'
    )


class CIE(_SkyWithSun):
    """CIE sky similar to using Radiance's gensky command."""

    type: constr(regex='^CIE$') = 'CIE'

    sky_type: int = Field(
        0,
        ge=0,
        le=5,
        description='An integer between 0..5 to indicate CIE Sky Type.'
        '\n0 = Sunny with sun.'
        '\n1 = Sunny without sun.'
        '\n2 = Intermediate with sun.'
        '\n3 = Intermediate without sun.'
        '\n4 = Cloudy sky.'
        '\n5 = Uniform cloudy sky.'
    )


class ClimateBased(_SkyWithSun):
    """Point-in-time Climate-based sky."""

    type: constr(regex='^ClimateBased$') = 'ClimateBased'

    direct_normal_irradiance: float = Field(
        ...,
        ge=0,
        description='Direct normal irradiance (W/m2).'
    )

    diffuse_horizontal_irradiance: float = Field(
        ...,
        ge=0,
        description='Diffuse horizontal irradiance (W/m2).'
    )


class SunMatrix(NoExtraBaseModel):
    """Annual Climate-based Sun matrix."""

    type: constr(regex='^SunMatrix$') = 'SunMatrix'

    wea: Wea = Field(
        ...,
        description='A Ladybug wea schema.'
    )

    north: float = Field(
        0,
        ge=-360,
        le=360,
        description='A number between -360 and 360 for the counterclockwise '
        'difference between the North and the positive Y-axis in degrees. '
        '90 is West and 270 is East.'
    )


class SkyMatrix(SunMatrix):
    """Annual Climate-based Sky matrix."""

    type: constr(regex='^SkyMatrix$') = 'SkyMatrix'

    density: int = Field(
        1,
        ge=1,
        description='Sky patch subdivision density. This values is similar to '
        '-m option in gendaymtx command. Default is 1 which means 145 sky patches '
        'and 1 patch for the ground. One can add to the resolution typically by '
        'factors of two (2, 4, 8, ...) which yields a higher resolution sky using '
        'the Reinhart patch subdivision.'
    )
